#include "sillythreads.h"

#include <unistd.h>

#include <QDebug>
#include <QImage>

#include <poppler-qt4.h>

SillyThread::SillyThread(Poppler::Document* document, QObject* parent) : QThread(parent),
    m_document(document),
    m_pages()
{
    m_pages.reserve(m_document->numPages());

    for(int index = 0; index < m_document->numPages(); ++index)
    {
        Poppler::Page* page = m_document->page(index);

        if(page == 0)
        {
            qDebug() << "!Document::page";
            exit(EXIT_FAILURE);
        }

        m_pages.append(page);
    }
}


void SillyThread::run()
{
    forever
    {
        foreach(Poppler::Page* page, m_pages)
        {
            QImage image = page->renderToImage();

            if(image.isNull())
            {
                qDebug() << "!Page::renderToImage";
                exit(EXIT_FAILURE);
            }
        }
    }
}


CrazyThread::CrazyThread(Poppler::Document* document, QObject* parent) : QThread(parent),
    m_document(document),
    m_page(0),
    m_annotation(0)
{
}

void CrazyThread::run()
{
    qsrand(static_cast< uint >(currentThreadId()));

    forever
    {
        if(qrand() % 2 == 0)
        {
            qDebug() << "search...";

            Poppler::Page* page = randomPage();

            page->search("c", Poppler::Page::CaseInsensitive);
            page->search("r", Poppler::Page::CaseSensitive);
            page->search("a", Poppler::Page::CaseInsensitive);
            page->search("z", Poppler::Page::CaseSensitive);
            page->search("y", Poppler::Page::CaseInsensitive);

            delete page;
        }

        if(qrand() % 2 == 0)
        {
            if(m_page == 0)
            {
                qDebug() << "add annotation...";

                m_page = randomPage();
                m_annotation = new Poppler::TextAnnotation(Poppler::TextAnnotation::Linked);

                m_annotation->setBoundary(QRectF(0.0, 0.0, 1.0, 1.0));

                m_annotation->setContents("crazy");
                m_page->addAnnotation(m_annotation);
            }
            else
            {
                qDebug() << "remove annotation...";

                m_annotation->setContents("CRAZY");
                m_page->removeAnnotation(m_annotation);

                delete m_page;

                m_page = 0;
                m_annotation = 0;
            }
        }

        if(qrand() % 2 == 0)
        {
            qDebug() << "fonts...";

            m_document->fonts();
        }
    }
}

Poppler::Page* CrazyThread::randomPage()
{
    Poppler::Page* page = m_document->page(qrand() % m_document->numPages());

    if(page == 0)
    {
        qDebug() << "!Document::page";
        exit(EXIT_FAILURE);
    }

    return page;
}

int main(int argc, char** argv)
{
    if(argc != 3)
    {
        qDebug() << "usage: sillythreads filePath duration";
        exit(EXIT_FAILURE);
    }

    QString filePath(argv[1]);
    int duration = atoi(argv[2]);

    Poppler::Document* document = Poppler::Document::load(filePath);

    if(document == 0)
    {
        qDebug() << "!Document::load";
        exit(EXIT_FAILURE);
    }

    (new SillyThread(document))->start();
    (new SillyThread(document))->start();

    (new CrazyThread(document))->start();
    (new CrazyThread(document))->start();

    sleep(duration);

    return EXIT_SUCCESS;
}
